# ESWIN DSP 自定义算子开发说明

```mermaid
flowchart TD
    X[Cadence-xtensa Environment Setup]
    A[ESWIN Cross-compilation Environment Setup]
    B[Ubuntu 20.04 Environment]
    B1[Get Docker and es_debian Image]
    C[Ubuntu 22.04 Environment]
    C1[Download eswin-sdk and extract files]
    D[Custom Operator Development]
    E[Operator Compilation]
    F[Generated Kernel Binary]
    G[Develop Test Case]
    G1[Compile Test Case]
    R[Create ESWIN rootfs on x86]
    DVB[ESWIN DVB]
    RUN[Run dsp_kernel_test with options]

    A --> B --> B1 --> R
    A --> C --> C1 --> R
    X --> E
    D --> E --> F --> DVB
    R --> E
    R --> G1
    G --> G1 --> DVB
    DVB --> RUN
```

## ESWIN 交叉编译环境搭建

用户编译环境分20.04和22.04两种，区别在于生成交叉编译目录rootfs，20.04只能用eswin提供的docker编译生成rootfs，22.04可以直接在x86编译生成rootfs。

以下都操作都基于ubuntu 20.04。

### 获取docker

从ESWIN获取es_debian docker image。

### 获取eswin-sdk-20241024.tar.gz

从ESWIN获取eswin-sdk-20241024.tar.gz（或最新版本）。

### 编译eswin交叉编译环境

- **步骤一**：参考《软件入门指南_CN_v1.3.docx》中`3.3 系统编译`章节，并执行到`make_all`命令完成。

- **步骤二**：Host执行
```shell {.line-numbers}
cd eswin-sdk-20241024/
sudo chmod 777 -R EIDS200B/
cd EIDS200B/output
mkdir rootfs
sudo apt-get install e2fsprogs
e2fsck -f root-EIDS200B-20241025-020530.ext4
resize2fs root-EIDS200B-20241025-020530.ext4 12G
sudo mount root-EIDS200B-20241025-020530.ext4 rootfs/

sudo chroot rootfs/
```
- **步骤三**：rootfs执行
```shell {.line-numbers}
root@ibudev12:/# cd home/eswin/
root@ibudev12:/# apt install -y gcc build-essential cmake
root@ibudev12:/# apt install es-sdk-dsp
root@ibudev12:/# apt install es-sdk-sample-dsp
root@ibudev12:/# apt install es-sdk-common
root@ibudev12:/# cd /usr/include/
root@ibudev12:/usr/include# ln -s riscv64-linux-gnu/bits bits
root@ibudev12:/usr/include# ln -s riscv64-linux-gnu/sys sys
root@ibudev12:/usr/include# ln -s riscv64-linux-gnu/gnu gnu
root@ibudev12:/usr/include# ln -s riscv64-linux-gnu/asm asm
root@ibudev12:/usr/include# cd /usr/lib
root@ibudev12:/usr/lib# cp riscv64-linux-gnu/* ./ -rf
root@ibudev12:/usr/lib# exit
```

### 获取riscv交叉编译工具

```shell {.line-numbers}
wget https://github.com/riscv-collab/riscv-gnu-toolchain/releases/download/2024.04.12/riscv64-glibc-ubuntu-20.04-gcc-nightly-2024.04.12-nightly.tar.gz
sudo tar -xvf riscv64-glibc-ubuntu-20.04-gcc-nightly-2024.04.12-nightly.tar.gz -C /opt
sudo cp /opt/riscv/sysroot/usr/lib/crt1.o eswin-sdk-20241024/EIDS200B/output/rootfs/usr/lib
```

### 拷贝sample code到任意目录

```shell {.line-numbers}
cp -rf /path/to/your/eswin-sdk-20241024/EIDS200B/output/rootfs/opt/eswin/sample-code/dsp_sample /path/to/your/target/dir
```

## Cadence Vision Q7 DSP 编译环境搭建

EIC7700集成了4个DSP核，使用的是Cadence公司的Vision Q7 DSP IP。
Cadence DSP开发环境安装可参考Cadence提供的官方文档《dev_tools_install_guide.pdf》。
注意：DSP开发依赖Cadence提供的集成开发环境，需要正确设置license之后才可以使用Xtensa Develop Tools，EIC7700不提供Cadence license，需要用户单独购买。


## 自定义算子开发

### 算子开发

```shell {.line-numbers}
cd /path/to/your/dsp_sample/kernel_sample/kernels/user_add/
// coding
```

#### host 侧准备

用户需要在 host 完成 op desc 生成、inputs 和 outputs 的内存申请和数据加载即可，其余步骤一般无需做修改。
1. op desc 填充方法参考 test/src/dsp_kernel_test.c 的 prepare_add_op_desc() 函数实现；
2. inputs 的内存申请和数据加载参考 test/src/dsp_kernel_test.c 的 loadInputsData() 函数实现；
3. outputs 的内存申请参考 test/src/dsp_kernel_test.c 的 prepareOutputBuffer() 函数实现。

#### dsp 侧算子开发

- **prepare、eval、get_prepare、get_eval**

上述 4 个函数的函数名、参数为固定接口，不能被修改。同时 prepare、get_prepare、get_eval 这 3 个函数的内容也不能被修改。
用户只需要在 eval 函数中完成自己算子的实现代码即可

```shell {.line-numbers}
ES_S32 eval(es_dsp_buffer_group *params, es_dsp_buffer_group *inputs, es_dsp_buffer_group *outputs)
{
    // do op eval
    return ES_DSP_SUCCESS;
}
```

eval 函数的入参为 3 个 es_dsp_buffer_group 结构体指针，分别存放的是算子的配置参数指针、输入 tensor 指针、输出 tensor 指针。
具体读取方法如下：

- **读取配置参数**

假设在 host 侧将 add 算子的 ADD_CONFIG_S 结构体存放到 ES_DSP_OPERATOR_DESC_S 结构体后，则可以使用：
```shell {.line-numbers}
ADD_CONFIG_S *config_ddr = (ADD_CONFIG_S *)(params->buffers[0].addr); 
```
从 params 读取算子配置的 ADD_CONFIG_S 信息。

- **读取输入/输出 tensor 指针**

同理 ifmap 和 ofmap 外部传入的指针，也可以通过该方法获取，若使用：
```shell {.line-numbers}
int8_t *ifmap1_ddr_ptr = (int8_t *)inputs->buffers[1].addr;
``` 
即可获取外部传入的第 2 个输入 tensor 的数据起始地址。

- **开始算子开发**

用户获取到算子配置信息和输入/输出 tensor 指针后，即可开始自己的算子开发。

##### 常见接口使用说明

- **获取 dram 的起始地址和大小**

```shell {.line-numbers}
char *dram0_base_ptr = (char *)((ES_U32)get_available_dram0_base());
ES_U32 dram0_size = get_available_dram0_size();
``` 

`注意：为了性能，请将算子的 config 搬运到 dram 上，并且将 ifmap 和 ofmap 分块放到 dram 上计算。指令从 dram 取数的速度比从 ddr 取数的速度快 10+ 倍。`

- **获取 spad 的起始地址和大小**

spad 内存空间是 framework 申请的 DDR 内存块，可以直接使用（在 DSP 内核端，不允许使用 malloc 等方法申请内存）

```shell {.line-numbers}
char *spad_base_ptr = (char *)((ES_U32)get_available_spad_base());
ES_U32 spad_size = get_available_spad_size();
``` 

- **获取程序执行时间戳**

```shell {.line-numbers}
    ES_S32 time_stamp;
#pragma no_reorder
    TIME_STAMP(time_stamp);
#pragma no_reorder
```

两个 TIME_STAMP 接口获取到的数值差值，单位为 cycle，EIC770X 芯片使用的 cadence Q7 dsp 芯片的频率为 1G，所以二者差值单位可以认为是 ns。可以通过该时间戳，做简单的性能统计

- **发起 dma 拷贝**

```shell {.line-numbers}
idma_copy_3d_desc64(CHANNEL, &dst_ptr, &src_ptr, IDMA_DISABLE_INT, row_sz, nrows, ntiles,
                            src_row_pitch, dst_row_pitch, src_tile_pitch, dst_tile_pitch);
```

`注：`
1. idma_copy_3d_desc64() 是 dma 异步拷贝的接口，该接口可以在进行计算的同时，在 ddr 和 dram 之间进行传输数据；
2. 该接口不会阻塞程序执行，即调用该接口后，具体的数据搬运会由 dma 模块接管，代码会向下继续执行；
3. 具体参数如下所示：
- **CHANNEL**：表示使用 idma 的通道编号(可选项为：0、1、2、3)，用户可以自由选用，不同通道没有性能差异；
- **dst_ptr，src_ptr**：分为表示待搬运数的目的地地址和源地址，以将 ddr --> dram 的数据搬运为例，dst_ptr 为 dram 上的地址，src_ptr 为 ddr 上原始数据的地址；
- **IDMA_DISABLE_INT**：为 dma 搬运的标记符，保持不变即可；
- **row_sz** 拷贝行大小：表示的是从指针现在所指的位置每次连续取值的长度；
- **nrows** 单次拷贝的重复次数：前面取 row_sz个数，然后指针偏移 src_row_pitch 距离再取 row_sz 个数。这样操作的重复次数；
- **ntiles**：完成上述 nrows 次的拷贝后，指针偏移src_tile_pitch，再完成下一个回合。这样重复 nrows  的次数；
- **src_row_pitch** 拷贝源 stride：表示当在指针位置连续取 row_sz 个数后，指针偏移量(下次取值初始位置的指针距离上一次取值初始位置的偏移量)；
- **dst_row_pitch** 目的地 stride：将拷贝完的这一行放在 dst_ptr 后，下一行距离这一行开头的偏移量，因为目标数据一般紧密排放，所以一般等于 row_sz；
- **src_tile_pitch** 拷贝源的面 stride：表示当完成前面 nrows 次重复拷贝 row_sz 个数(将这个集合称为一个回合)后，下一个回合指针起点和上个回合指针起点之间的偏移量；
- **dst_tile_pitch** 目的地的面 stride：拷贝完 nrows 次后，下一个存放通道到上一个存放通道的距离。

- **等待 dma 拷贝结束**

```shell {.line-numbers}
while (idma_hw_num_outstanding(CHANNEL) > 0) {
}
```

等待 CHANNEL 通道的 dma 拷贝结束，`注意：该接口会阻塞整体程序执行，代码会等待这个 while 循环退出再进行下一个步骤。`

- **日志打印**
```shell {.line-numbers}
es_printf("this is eval of add op\n");
```
在 dsp 上打印日志，请使用 es_printf 函数进行日志打印。  
注意：  
1、请登录串口，查看dsp侧的打印日志，例如 (下面命令仅供参考，具体命令请根据自身开发板连接方式进行调整)：
```shell {.line-numbers}
ssh 192.168.1.100 -l hostname
输入密码：password
sudo hostname -D /dev/ttyUSB3
```
登录串口后，再执行可执行程序调用算子，即可在串口获取到es_printf 打印的内容。  
2、在dsp侧使用 es_printf打印日志，会对算子性能造成较大影响。根据打印的日志长度，大致会导致耗时增加数十~数百微妙。
- **算子具体的实现过程**

算子具体的实现过程，一般是使用 dsp 的 SIMD 指令对数据进行并行计算，这部分的指令使用思想可以参考 x86 的 SSE/AVX、arm 的 neon 指令集。

### 算子编译

算子编译前，根据实际情况更改 `dsp_sample/kernel_sample/kernels/build.sh` 函数 `xtensa_env_setup` 中的 `xtensa配置`。

```shell {.line-numbers}
function xtensa_env_setup() {
    XTENSA_CORE="dsp_base648229_1108"
    XTENSA_TOOLS_VERSION="RI-2021.7-linux"
    XTENSA_DEVTOOLS_ROOT="/opt/Cadence_EDA_Tools/Xtensa_DSP_Q7/XtDevTools"
    XTENSA_BUILDS_ROOT="${XTENSA_DEVTOOLS_ROOT}/install/builds/${XTENSA_TOOLS_VERSION}/${XTENSA_CORE}"
    XTENSA_TOOLS_ROOT="${XTENSA_DEVTOOLS_ROOT}/install/tools/${XTENSA_TOOLS_VERSION}/XtensaTools"
    export XTENSA_ARC_INCLUDE="${XTENSA_BUILDS_ROOT}/xtensa-elf/arch/include"
    export XTENSA_TOOLS_BIN="${XTENSA_TOOLS_ROOT}/bin"
    export XTENSA_ELF_INCLUDE="${XTENSA_TOOLS_ROOT}/xtensa-elf/include"
    export XTENSA_ELF_SRC="${XTENSA_TOOLS_ROOT}/xtensa-elf/src"
    export XTENSA_TIE_INCLUDE="${XTENSA_ARC_INCLUDE}/xtensa/tie"
    export XTENSA_CONFIG_INCLUDE="${XTENSA_ARC_INCLUDE}/xtensa/config"
    export XTENSA_LIBXI_INSTALL="${XTENSA_ELF_SRC}/xocl/Tests/libxi"
    export XTENSA_KERNEL_LSP="${kernel_sample_dir}/include/xtensa/mloader/pimultload"
    export XTENSA_COMPLIER=${XTENSA_TOOLS_BIN}/xt-clang
    export XTENSA_COMPLIER_FLAGS="--xtensa-core=${XTENSA_CORE} ${gcc_optimization} -Werror -mlongcalls -fmessage-length=0"
}
```

`注意：由于XTENSA_CORE和XTENSA_TOOLS_VERSION可能存在差异，如编译遇到问题请及时沟通解决。`

```shell {.line-numbers}
cd /path/to/your/dsp_sample/kernel_sample/kernels/
./build.sh /path/to/your/eswin-sdk-20241024/EIDS200B/output/rootfs
```

## 自定义算子测试

### 测试用例

```shell {.line-numbers}
cd /path/to/your/dsp_sample/kernel_sample/test
// coding
```

### 编译

```shell {.line-numbers}
cd /path/to/your/dsp_sample/kernel_sample/test
./build.sh /path/to/your/eswin-sdk-20241024/EIDS200B/output/rootfs /path/to/your/kernel_sample/kernels/user_add
```

### 运行

- 将编译好的算子`es_dsp_kernel_add.pkg.lib`拷贝到开发板`/lib/firmware/dsp_kernels/`；
- 将测试程序`dsp_kernel_test`和测试文件`input_0.bin, input_1.bin`拷贝到开发板任意位置，如：`/opt/eswin/`；
- 执行测试命令：
```shell {.line-numbers}
cd /opt/eswin/
./dsp_kernel_test -c 0 -i input_0.bin input_1.bin
```
### 正确性验证

执行结束后，会在`input_0.bin`所在目录，生成`output.bin`。将其拷出到Linux下，使用：
```shell {.line-numbers}
md5sum output.bin
```
查看其 md5 码，如果其值为：4aae41d770845ae89f0cf33be15d0a77，则表示结果正确。
