#!/bin/bash
set -e

sys_root_dir=null
kernel_sample_dir=$(dirname $(readlink -f "$0"))

function xtensa_env_setup() {
    XTENSA_CORE="dsp_base648229_1108"
    XTENSA_TOOLS_VERSION="RI-2021.7-linux"
    XTENSA_DEVTOOLS_ROOT="/opt/Cadence_EDA_Tools/Xtensa_DSP_Q7/XtDevTools"
    XTENSA_BUILDS_ROOT="${XTENSA_DEVTOOLS_ROOT}/install/builds/${XTENSA_TOOLS_VERSION}/${XTENSA_CORE}"
    XTENSA_TOOLS_ROOT="${XTENSA_DEVTOOLS_ROOT}/install/tools/${XTENSA_TOOLS_VERSION}/XtensaTools"
    export XTENSA_ARC_INCLUDE="${XTENSA_BUILDS_ROOT}/xtensa-elf/arch/include"
    export XTENSA_TOOLS_BIN="${XTENSA_TOOLS_ROOT}/bin"
    export XTENSA_ELF_INCLUDE="${XTENSA_TOOLS_ROOT}/xtensa-elf/include"
    export XTENSA_ELF_SRC="${XTENSA_TOOLS_ROOT}/xtensa-elf/src"
    export XTENSA_TIE_INCLUDE="${XTENSA_ARC_INCLUDE}/xtensa/tie"
    export XTENSA_CONFIG_INCLUDE="${XTENSA_ARC_INCLUDE}/xtensa/config"
    export XTENSA_LIBXI_INSTALL="${XTENSA_ELF_SRC}/xocl/Tests/libxi"
    export XTENSA_KERNEL_LSP="${kernel_sample_dir}/include/xtensa/mloader/pimultload"
    export XTENSA_COMPLIER=${XTENSA_TOOLS_BIN}/xt-clang
    export XTENSA_COMPLIER_FLAGS="--xtensa-core=${XTENSA_CORE} -O2 -Werror -mlongcalls -fmessage-length=0"
}

function eswin_env_setup() {
    export ESWIN_SDK_INCLUDE=${sys_root_dir}/usr/include/essdk
}

function usage() {
    echo -e "Usage:\n$0 <sys_root_dir>"
}

function main() {
    if [ $# -le 0 ]; then
        usage
        exit 1
    fi

    sys_root_dir=$1

    if [ ! -d build ]; then
        mkdir -p build
    else
        rm -rf build/*
    fi

    cd build

    # set cadence environments
    xtensa_env_setup

    # set eswin environments
    eswin_env_setup

    # build dsp kernel
    cmake ../ -DCMAKE_VERBOSE_MAKEFILE=OFF
    make
}

main $@
