// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef ESWIN_DSP_COMMON_DSP_CB_DESC_H_
#define ESWIN_DSP_COMMON_DSP_CB_DESC_H_

#include <stdint.h>
#include <xtensa/idma.h>

typedef idma_status_t (*idma_init_pfunc)(int32_t, uint32_t, idma_max_block_t, uint32_t, idma_ticks_cyc_t, uint32_t,
                                         idma_err_callback_fn);

typedef idma_status_t (*idma_init_loop_pfunc)(int32_t, idma_buffer_t *, idma_type_t, int32_t, void *, idma_callback_fn);

typedef int32_t (*idma_copy_3d_desc64_pfunc)(int32_t, void *, void *, uint32_t, size_t, uint32_t, uint32_t, uint32_t,
                                             uint32_t, uint32_t, uint32_t);

typedef int32_t (*idma_copy_desc64_pfunc)(int32_t, void *, void *, uint32_t, uint32_t);

typedef uint32_t (*idma_hw_num_outstanding_pfunc)(int32_t);

typedef int32_t (*idma_desc_done_pfunc)(int32_t, int32_t);

typedef struct {
    idma_init_pfunc idma_init;
    idma_hw_num_outstanding_pfunc idma_hw_num_outstanding;
    idma_init_loop_pfunc idma_init_loop;
    idma_copy_desc64_pfunc idma_copy_desc64;
    idma_copy_3d_desc64_pfunc idma_copy_3d_desc64;
    idma_desc_done_pfunc idma_desc_done;
} idma_callback_if;

typedef int (*printf_pfunc)(const char *, ...);
typedef int (*snprintf_pfunc)(char *, uint32_t, const char *, ...);
typedef int (*vsnprintf_pfunc)(char *, uint32_t, const char *, ...);
typedef void (*abort_pfunc)();

typedef struct {
    printf_pfunc printf;
    snprintf_pfunc snprintf;
    vsnprintf_pfunc vsnprintf;
    abort_pfunc abort;
} std_callback_if;

typedef void (*dcache_pfunc)(void *, size_t);
typedef struct {
    dcache_pfunc dcache_flush;
    dcache_pfunc dcache_invalidate;
    dcache_pfunc dcache_flush_invalidate;
} cache_callback_if;

typedef struct {
    std_callback_if std_cb_if;
    idma_callback_if idma_cb_if;
    cache_callback_if cache_cb_if;
} dsp_callback_if;

typedef struct {
    dsp_callback_if cb_if;
} dsp_common_if;
#endif  // ESWIN_DSP_COMMON_DSP_CB_DESC_H_
