// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef ESWIN_DSP_COMMON_DSP_CB_IF_H_
#define ESWIN_DSP_COMMON_DSP_CB_IF_H_
#include "dsp_cb_desc.h"
#include "dsp_ram.h"

#define IDMA_BUFF_SIZE 0x42d0                              /* idma init need this dram */
#define CB_IF_ADDR (XCHAL_DATARAM0_VADDR + IDMA_BUFF_SIZE) /* consist addr for callback interface */

static inline dsp_callback_if *get_cb_if() { return (dsp_callback_if *)CB_IF_ADDR; }

/**
 * @brief  The wrapper function of idma_hw_num_outstanding() in idma
 *             NOTE: more detail info in idma.h
 */
static inline uint32_t es_idma_hw_num_outstanding(int32_t ch)
{
    return get_cb_if()->idma_cb_if.idma_hw_num_outstanding(ch);
}

/**
 * @brief  The wrapper function of idma_desc_done() in idma
 */
static inline int32_t es_idma_desc_done(int32_t ch, int32_t index)
{
    return get_cb_if()->idma_cb_if.idma_desc_done(ch, index);
}

/**
 * @brief  The wrapper function of idma_copy_desc64() in idma
 */
static inline int32_t es_idma_copy_desc64(int32_t ch, void *dst, void *src, size_t size, uint32_t flags)
{
    return get_cb_if()->idma_cb_if.idma_copy_desc64(ch, dst, src, size, flags);
}

/**
 * @brief  The wrapper function of idma_copy_3d_desc64() in idma
 */
static inline int32_t es_idma_copy_3d_desc64(int32_t ch, void *dst, void *src, uint32_t flags, uint32_t row_sz,
                                             uint32_t nrows, uint32_t ntiles, uint32_t src_row_pitch,
                                             uint32_t dst_row_pitch, uint32_t src_tile_pitch, uint32_t dst_tile_pitch)
{
    return get_cb_if()->idma_cb_if.idma_copy_3d_desc64(ch, dst, src, flags, row_sz, nrows, ntiles, src_row_pitch,
                                                       dst_row_pitch, src_tile_pitch, dst_tile_pitch);
}

static inline void dsp_dcache_flush(void *addr, size_t len) { get_cb_if()->cache_cb_if.dcache_flush(addr, len); }

static inline void dsp_dcache_invalidate(void *addr, size_t len)
{
    get_cb_if()->cache_cb_if.dcache_invalidate(addr, len);
}

static inline void dsp_dcache_flush_invalidate(void *addr, size_t len)
{
    get_cb_if()->cache_cb_if.dcache_flush_invalidate(addr, len);
}

/**
 * @brief  The wrapper function of printf() in stdio
 */
#define es_printf(format, ...) get_cb_if()->std_cb_if.printf(format, ##__VA_ARGS__)

/**
 * @brief The Assert function
 */
#define assert(condition)                                                                                    \
    {                                                                                                        \
        if (!condition) {                                                                                    \
            get_cb_if()->std_cb_if.printf("%s:%d %s -- assertion failed\n", __FILE__, __LINE__, #condition); \
            get_cb_if()->std_cb_if.abort();                                                                  \
        }                                                                                                    \
    }
#endif  // ESWIN_DSP_COMMON_DSP_CB_IF_H_
