// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef ESWIN_DSP_COMMON_DSP_RAM_H_
#define ESWIN_DSP_COMMON_DSP_RAM_H_

#include <xtensa/config/core.h>
#include "es_dsp_types.h"

/**
 * @brief Get the byte offset of an member relative to the start of a given data type.
 *
 */
#define ADDR_OFFSET(type, member) ((char*)(&((type*)(NULL))->member) - (char*)NULL)

#define __DATARAM0__ __attribute__((section(".dram0.data")))
#define __DATARAM1__ __attribute__((section(".dram1.data")))

#define SPAD_ADDR_START 0xff9b0000  // come from sram_3 of memmap.xmm
#define SPAD_ADDR_END 0xfffb0000    // come from sram_3 of memmap.xmm

#define E31_BOOT_SIGN_OFFSET (0x8)

#define ES_DATARAM0_VADDR XCHAL_DATARAM0_VADDR
#define ES_DATARAM0_SIZE XCHAL_DATARAM0_SIZE
#define ES_DATARAM1_VADDR XCHAL_DATARAM1_VADDR
#define ES_DATARAM1_SIZE XCHAL_DATARAM1_SIZE

extern char __dram0_data_end;
extern char __dram0_data_start;

extern char __dram1_data_end;
extern char __dram1_data_start;

extern char __data_seg_start;
extern char __data_seg_end;

#define DATARAM_ALIGN(n) (((n) + 0x3f) & (~0x3f))
#define DATARAM0_OFFSET DATARAM_ALIGN((&__dram0_data_end - &__dram0_data_start) * sizeof(char))
#define DATARAM1_OFFSET DATARAM_ALIGN((&__dram1_data_end - &__dram1_data_start) * sizeof(char))
#define DATASPAD_OFFSET ((&__data_seg_end - &__data_seg_start) * sizeof(char))

#define RESERVED_DATARAM0_SIZE (20 * 1024)  // reserved size for firmware
#define RESERVED_DATARAM1_SIZE (8 * 1024)   // reserved size for firmware stack
/**
 * @brief Get the available beginning address from dram0
 *
 * @return ES_U32 The available address
 */
static inline ES_U32 get_available_dram0_base() { return ES_DATARAM0_VADDR + DATARAM0_OFFSET + RESERVED_DATARAM0_SIZE; }

/**
 * @brief Get the available size of dsp dram0
 *
 * @return ES_S32 The available size
 */
static inline ES_S32 get_available_dram0_size() { return ES_DATARAM0_SIZE - RESERVED_DATARAM0_SIZE - DATARAM0_OFFSET; }

/**
 * @brief Get the available begin address of dsp dram1
 *
 * @return ES_U32 The available address
 */
static inline ES_U32 get_available_dram1_base() { return ES_DATARAM1_VADDR + DATARAM1_OFFSET; }

/**
 * @brief Get the available size of dsp dram1
 *
 * @return ES_S32 The available size
 */
static inline ES_S32 get_available_dram1_size() { return ES_DATARAM1_SIZE - DATARAM1_OFFSET - RESERVED_DATARAM1_SIZE; }

/**
 * @brief Get the available beginning spad address
 *
 * @return ES_U32 The available address
 */
static inline ES_U32 get_available_spad_base() { return SPAD_ADDR_START + DATASPAD_OFFSET; }

/**
 * @brief Get the available spad size
 *
 * @return ES_S32 The available size
 */
static inline ES_S32 get_available_spad_size() { return SPAD_ADDR_END - SPAD_ADDR_START - DATASPAD_OFFSET; }

#endif  // ESWIN_DSP_COMMON_DSP_RAM_H_
