// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef ESWIN_DSP_KERNEL_COMMON_ES_KERNEL_PUBLIC_DEFINES_H_
#define ESWIN_DSP_KERNEL_COMMON_ES_KERNEL_PUBLIC_DEFINES_H_

#include "stdint.h"
#include "es_nn_common.h"
#include "es_dsp_error.h"
//#include "es_pipo_public/defines.h"

#define S16_MIN (-32768)
#define FP16_MIN (-65504)
#define FP32_MIN (-2147483648.0f)

typedef struct _tiling_info {
    ES_U32 tile_n;
    ES_U32 tile_c;
    ES_U32 tile_h;
    ES_U32 tile_w;
    ES_U32 tile_c0;
    ES_U32 data_size_id;  // 0-fast  1-mid  2-slow
} __attribute__((packed)) tiling_info;

typedef struct _es_dsp_tensor {
    void* __attribute((aligned(8))) p_data;
    ES_TENSOR_SHAPE_S __attribute((aligned(8))) shape;
    ES_DATA_PRECISION_E __attribute((aligned(8))) data_type;
} __attribute((aligned(8))) es_dsp_tensor;

#define EPSILON 1e-6 /* 定义一个极小值，用于比较浮点数的大小差异 */

#define LOCAL_MEMORY_SIZE (96 * 1024)  // dram generally allows for the size of tensor

#if defined PROFILING_ENABLE && PROFILING_ENABLE
#define PRINT_TIME(start, end, name)                                  \
    {                                                                 \
        es_printf("time in %s cost : %d cycle\n", name, end - start); \
    }
#define PRINT_KERNEL_CYCLE                                                                                \
    {                                                                                                     \
        es_printf("All:%d \n", ping_pong_cycles.init_used_cycles + ping_pong_cycles.process_used_cycles); \
        es_printf("Init use:%d \n", ping_pong_cycles.init_used_cycles);                                   \
        es_printf("Process use: %d \n", ping_pong_cycles.process_used_cycles);                            \
        es_printf("proc ping:%d/%d proc pong:%d/%d  \n", ping_pong_cycles.ping_process_accumulate,        \
                  ping_pong_cycles.ping_process_accumulate_num, ping_pong_cycles.pong_process_accumulate, \
                  ping_pong_cycles.pong_process_accumulate_num);                                          \
        for (int i = 0; i < TILENUM; ++i) {                                                               \
            es_printf("pingpong DMA Ch0: %d , Ch1: %d , Ch2: %d , Ch3: %d , pingpong kernel: %d\n",       \
                      ping_pong_cycles.cycles_for_chns[0][i], ping_pong_cycles.cycles_for_chns[1][i],     \
                      ping_pong_cycles.cycles_for_chns[2][i], ping_pong_cycles.cycles_for_chns[3][i],     \
                      ping_pong_cycles.cycles_for_kernel[i]);                                             \
        }                                                                                                 \
    }
#else
#define PRINT_TIME(start, end, name) \
    {                                \
    }
#define PRINT_KERNEL_CYCLE ;
#endif

// #define ENABLE_DEBUG_PRINT
#if defined BUILD_RELEASE && BUILD_RELEASE
#define ES_KERNEL_LOG(format, ...) \
    {                              \
    }
#else
#define ES_KERNEL_LOG(format, args...) es_printf(format, args);
#endif

#if defined BUILD_RELEASE && BUILD_RELEASE
#define ES_LOG_ARRAY(x, num, TAG) \
    {                             \
    }
#else
#define ES_LOG_ARRAY(x, num, TAG)                  \
    ES_KERNEL_LOG("%s: ", TAG);                    \
    for (int i = 0; i < num; ++i) {                \
        if (i % 8 == 0) ES_KERNEL_LOG("\n", NULL); \
        ES_KERNEL_LOG(" %f ", x[i]);               \
    }                                              \
    ES_KERNEL_LOG("\n", NULL);
#endif

#ifdef XPLORER_SIM
#include "dsp_hw_if.h"
#define XPLORE_HOOK(name)                          \
    void register_op_##name(operator_func* opfunc) \
    {                                              \
        opfunc->eval_func = eval;                  \
        opfunc->prepare_func = prepare;            \
    }
#define XPLORE_HOOK_DECLARE(name) extern void register_op_##name(operator_func* opfunc);
#else
#define XPLORE_HOOK(name)
#define XPLORE_HOOK_DECLARE(name)
#endif

// #define ENABLE_DBG_WRITE
#ifdef ENABLE_DBG_WRITE
#include "stdio.h"
static ES_S32 write_bin(const char* filename, const ES_U64 size, const char* buf)
{
    FILE* file_p = NULL;

    file_p = fopen(filename, "w");
    if (file_p == NULL) {
        return ES_DSP_ERROR_OPEN_FILE_FAILED;
    }

    fwrite((void*)buf, 1, size, file_p);
    fclose(file_p);
    return ES_DSP_SUCCESS;
}
#endif

#endif  // ESWIN_DSP_KERNEL_COMMON_ES_KERNEL_PUBLIC_DEFINES_H_
