/*
 * Copyright (c) 2020 by Cadence Design Systems, Inc.  ALL RIGHTS RESERVED.
 * These coded instructions, statements, and computer programs are the
 * copyrighted works and confidential proprietary information of
 * Cadence Design Systems Inc.  They may be adapted and modified by bona fide
 * purchasers for internal use, but neither the original nor any adapted
 * or modified version may be disclosed or distributed to third parties
 * in any manner, medium, or form, in whole or in part, without the prior
 * written consent of Cadence Design Systems Inc.  This software and its
 * derivatives are to be executed solely on products incorporating a Cadence
 * Design Systems processor.
 */

/* *****************************************************************************
 * FILE:  defines.h
 *
 * DESCRIPTION:
 *
 *    This file contains prototype definitions of functions, structures and Macros related to
 *    scatter-gather basic routines that are being used/accessed in other project files
 *
 * ****************************************************************************/

#ifndef ESWIN_DSP_KERNEL_COMMON_ES_PIPO_PUBLIC_DEFINES_H_
#define ESWIN_DSP_KERNEL_COMMON_ES_PIPO_PUBLIC_DEFINES_H_

#if defined(__XTENSA__)
#include <xtensa/xt_profiling.h>
#endif
#include <xtensa/tie/xt_ivpn.h>
#include <xtensa/tie/xt_timer.h>

#if defined(__XTENSA__)
#define TIME_STAMP(cyc_cnt)        \
    {                              \
        cyc_cnt = XT_RSR_CCOUNT(); \
    }

#define USER_DEFINED_HOOKS_START()                 \
    {                                              \
        xt_iss_switch_mode(XT_ISS_CYCLE_ACCURATE); \
        xt_iss_trace_level(3);                     \
        xt_iss_client_command("all", "enable");    \
    }

#define USER_DEFINED_HOOKS_STOP()                \
    {                                            \
        xt_iss_switch_mode(XT_ISS_FUNCTIONAL);   \
        xt_iss_trace_level(0);                   \
        xt_iss_client_command("all", "disable"); \
    }

#else  // __XTENSA__
#define TIME_STAMP(cyc_cnt) \
    {                       \
        cyc_cnt = 0;        \
    }

#define USER_DEFINED_HOOKS_START()

#define USER_DEFINED_HOOKS_STOP()

#endif  // __XTENSA__

// the following three macro functions be used for dsp op internal time stamp record and print
#define KB(n) ((1 << 10) * n)

// provide default values
#ifndef DRAM0_TIME_STAMP_OFFSET
#define DRAM0_TIME_STAMP_OFFSET (100 * 1024)
#endif

#ifndef MAX_RECORD_TIME_CNT
#define MAX_RECORD_TIME_CNT (600)
#endif

#define INIT_TIME_STAMP()                                                                       \
    {                                                                                           \
        int32_t *time_info = (int32_t *)(get_available_dram0_base() + DRAM0_TIME_STAMP_OFFSET); \
        *time_info = 0;                                                                         \
    }

#define RECORD_TIME_STAMP(insert_indices)                                                       \
    {                                                                                           \
        int32_t *time_info = (int32_t *)(get_available_dram0_base() + DRAM0_TIME_STAMP_OFFSET); \
        if ((*time_info + 1) / 2 <= MAX_RECORD_TIME_CNT) {                                      \
            int32_t time_stamp;                                                                 \
            TIME_STAMP(time_stamp);                                                             \
            time_info[++*time_info] = insert_indices;                                           \
            time_info[++*time_info] = time_stamp;                                               \
        }                                                                                       \
    }

#define PRINT_TIME_STAMP()                                                                                        \
    {                                                                                                             \
        int32_t *time_info = (int32_t *)(get_available_dram0_base() + DRAM0_TIME_STAMP_OFFSET);                   \
        es_printf("the number of time stamp recorded is %d\n", (*time_info + 1) / 2);                             \
        int32_t timing_start = time_info[2];                                                                      \
        for (int32_t i = 1; i < *time_info; i += 2) {                                                             \
            es_printf("insert_indices is %d, time stamp is %d\n", time_info[i], time_info[i + 1] - timing_start); \
        }                                                                                                         \
    }

#endif  // ESWIN_DSP_KERNEL_COMMON_ES_PIPO_PUBLIC_DEFINES_H_
