// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#pragma once

#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>

#include "dsp_cb_if.h"
#include "dsp_ram.h"
#include "es_pingpong_macro.h"
#include "es_pingpong_interface.h"
#include "defines.h"

#define CHECK_ALLROLLIN_DONE(es_this, done_func, proc_func, chn) \
    if (!done_func(es_this)) {                                   \
        proc_func(es_this, chn);                                 \
    }

#define CHECK_ALLROLLOUT_DONE(es_this, done_func, proc_func) \
    if (!done_func(es_this)) {                               \
        proc_func(es_this);                                  \
    } else {                                                 \
        break;                                               \
    }

static int handle_ping(CLS_PTR)
{
    // launch copy pong in
    CHECK_ALLROLLIN_DONE(es_this, ROLLIN_DONE_FUNC, ROLLIN_FUNC, ROLLIN_PONG);
    WAIT_PING_FUNC(es_this);

    IVP_PINGPARAM_PROCESS();

    IVP_KERNEL_PROCESS();

    // launch copy ping out
    ROLLOUT_FUNC(es_this, ROLLOUT_PING);
    return 0;
}

static int handle_pong(CLS_PTR)
{
    // launch copy ping in
    CHECK_ALLROLLIN_DONE(es_this, ROLLIN_DONE_FUNC, ROLLIN_FUNC, ROLLIN_PING);
    WAIT_PONG_FUNC(es_this);

    IVP_PONGPARAM_PROCESS();

    IVP_KERNEL_PROCESS();

    // launch copy pong out
    ROLLOUT_FUNC(es_this, ROLLOUT_PONG);
    return 0;
}

static int process(CLS_PTR)
{
    // copy ping initially
    CHECK_ALLROLLIN_DONE(es_this, ROLLIN_DONE_FUNC, ROLLIN_FUNC, ROLLIN_PING);
    while (1) {
        CHECK_ALLROLLOUT_DONE(es_this, ROLLOUT_DONE_FUNC, handle_ping);
        CHECK_ALLROLLOUT_DONE(es_this, ROLLOUT_DONE_FUNC, handle_pong);
    }
    WAIT_TAIL_FUNC(es_this);

    return 0;
}

es_pingpong_processer* IVP_PINGPONG_CREATE_FUNC(CLS_PTR, es_pingpong_processer_config config)
{
    processor->init = (int (*)(void*, void*, void*, void*, void*))(config.init);
    processor->process = (int (*)(void*))process;

    es_this->config = config;
    es_this->config.process = (int (*)(void*))process;

    es_this->ping_start = (char*)get_available_dram0_base();
    es_this->pong_start = (char*)get_available_dram1_base();
    es_this->spad_start = (char*)get_available_spad_base();

    es_this->pipa_param = config.pipa_param;

    return processor;
}
