// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#pragma once

#include "stdio.h"
#include "stdlib.h"
#include "defines.h"

#define START 0
#define STOP 1
#define TILENUM 20

#define WAIT_IDMA(channel)                         \
    while (idma_hw_num_outstanding(channel) > 0) { \
    };

#define CLS_PTR es_pingpong_processer* processor
#define es_this processor

#define USING_4CHNS

typedef struct {
    int (*init)(void* proc, void* input_buffer_stru_, void* output_buffer_stru_, void* config_stru_,
                void* ivp_process_func_);

    int (*process)(void* proc);

    // variables
    void* pipa_param;
} es_pingpong_processer_config;

typedef struct {
    int32_t cycle_anchors[4];
    int32_t cycles_for_chns[4][TILENUM];
    int32_t cycles_for_kernel[TILENUM];
    int32_t init_used_cycles;
    int32_t process_used_cycles;
    int32_t ping_process_accumulate;
    int32_t ping_process_accumulate_num;
    int32_t pong_process_accumulate;
    int32_t pong_process_accumulate_num;
} PING_PONG_TIME_COUNT_S;

typedef struct {
    int (*init)(void* proc, void* in_buffer_info, void* out_buffer_info, void* config_struct, void* ivp_process_func_);

    int (*process)(void* proc);

    void* input_buffer_stru;
    void* output_buffer_stru;
    void* config_stru;

    PING_PONG_TIME_COUNT_S ping_pong_cycles;

    int32_t bank_size;
    int32_t spad_size;
    char* ping_start;
    char* pong_start;
    char* spad_start;

    void* ivp_process_func;
    void* pipa_param;

    es_pingpong_processer_config config;
    int32_t ping_tile_param_size;
    int32_t pong_tile_param_size;
} es_pingpong_processer;

static void mark_cycle_anchor(CLS_PTR, int32_t channel)
{
#pragma no_reorder
    TIME_STAMP(es_this->ping_pong_cycles.cycle_anchors[channel]);
#pragma no_reorder
}
