// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#pragma once

#include <stdio.h>
#include <stdlib.h>

#define ROLLIN_PING 0
#define ROLLIN_PONG 1
#define ROLLOUT_PING 0
#define ROLLOUT_PONG 1

#define idma_copy_desc64 es_idma_copy_desc64
#define idma_copy_3d_desc64 es_idma_copy_3d_desc64
#define idma_hw_num_outstanding es_idma_hw_num_outstanding
#define idma_desc_done es_idma_desc_done

#define DESC_NOTIFY_W_INT 0x80000000 /* trigger interrupt on completion */
#define DESC_IDMA_PRIOR_H 0x08000    /* QoS high */
#define IDMA_CHANNEL_0 0
#define IDMA_CHANNEL_1 1
#define IDMA_CHANNEL_2 2
#define IDMA_CHANNEL_3 3
#define IDMA_COPY_FLAG DESC_IDMA_PRIOR_H | DESC_NOTIFY_W_INT
#define IDMA_DISABLE_INT 0

#define PING 0
#define PONG 1

static void dma_3d_copy(void *pdst, void *psrc, int buffer_size)
{
    uint64_t srcw = (uint64_t)(uint32_t)psrc;
    uint64_t dstw = (uint64_t)(uint32_t)pdst;

    idma_copy_3d_desc64(IDMA_CHANNEL_0, &dstw, &srcw, IDMA_DISABLE_INT, buffer_size, 1, 1, 1, 1, 1, 1);
    while (idma_hw_num_outstanding(IDMA_CHANNEL_0) > 0) {
    }
}

static void dma_copy(void *pdst, void *psrc, int buffer_size)
{
    uint64_t srcw = (uint64_t)(uint32_t)psrc;
    uint64_t dstw = (uint64_t)(uint32_t)pdst;

    idma_copy_desc64(IDMA_CHANNEL_0, &dstw, &srcw, buffer_size, IDMA_DISABLE_INT);
    while (idma_hw_num_outstanding(IDMA_CHANNEL_0) > 0) {
    }
}

static void dma_copy_no_wait(int8_t ch, void *pdst, void *psrc, int buffer_size)
{
    uint64_t srcw = (uint64_t)(uint32_t)psrc;
    uint64_t dstw = (uint64_t)(uint32_t)pdst;

    idma_copy_desc64(ch, &dstw, &srcw, buffer_size, IDMA_DISABLE_INT);
}
