// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#pragma once
#pragma once

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define esfloat16 uint16_t

#ifndef float32_t
typedef float float32_t;
#endif

#ifndef float64_t
typedef double float64_t;
#endif

static int16_t float32_to_float16(float32_t value)
{
    // 1 : 8 : 23
    union {
        uint32_t u;
        float32_t f;
    } tmp;

    tmp.f = value;

    // 1 : 8 : 23
    uint16_t sign = (tmp.u & 0x80000000) >> 31;
    uint16_t exponent = (tmp.u & 0x7F800000) >> 23;
    uint32_t significand = tmp.u & 0x7FFFFF;

    //     NCNN_LOGE("%d %d %d", sign, exponent, significand);

    // 1 : 5 : 10
    int16_t fp16;
    if (exponent == 0) {
        // zero or denormal, always underflow
        fp16 = (sign << 15) | (0x00 << 10) | 0x00;
    } else if (exponent == 0xFF) {
        // infinity or NaN
        fp16 = (sign << 15) | (0x1F << 10) | (significand ? 0x200 : 0x00);
    } else {
        // normalized
        short newexp = exponent + (-127 + 15);
        if (newexp >= 31) {
            // overflow, return infinity
            fp16 = (sign << 15) | (0x1F << 10) | 0x00;
        } else if (newexp <= 0) {
            // Some normal fp32 cannot be expressed as normal fp16
            fp16 = (sign << 15) | (0x00 << 10) | 0x00;
        } else {
            // normal fp16
            fp16 = (sign << 15) | (newexp << 10) | (significand >> 13);
        }
    }

    return fp16;
}

static float32_t float16_to_float32(unsigned short value)
{
    // 1 : 5 : 10
    uint16_t sign = (value & 0x8000) >> 15;
    uint16_t exponent = (value & 0x7c00) >> 10;
    uint16_t significand = value & 0x03FF;

    //     NCNN_LOGE("%d %d %d", sign, exponent, significand);

    // 1 : 8 : 23
    union {
        uint32_t u;
        float32_t f;
    } tmp;
    if (exponent == 0) {
        if (significand == 0) {
            // zero
            tmp.u = (sign << 31);
        } else {
            // denormal
            exponent = 0;
            // find non-zero bit
            while ((significand & 0x200) == 0) {
                significand <<= 1;
                exponent++;
            }
            significand <<= 1;
            significand &= 0x3FF;
            tmp.u = (sign << 31) | ((-exponent + (-15 + 127)) << 23) | (significand << 13);
        }
    } else if (exponent == 0x1F) {
        // infinity or NaN
        tmp.u = (sign << 31) | (0xFF << 23) | (significand << 13);
    } else {
        // normalized
        tmp.u = (sign << 31) | ((exponent + (-15 + 127)) << 23) | (significand << 13);
    }

    return tmp.f;
}

#define MAX_NUM 32
static float32_t* vector_f16_to_f32(unsigned short* vector_f16_ptr, int num)
{
    static float32_t vector_f32_ptr[MAX_NUM];
    int max_num = num > MAX_NUM ? MAX_NUM : num;
    for (int i = 0; i < max_num; ++i) {
        vector_f32_ptr[i] = float16_to_float32(vector_f16_ptr[i]);
    }
    return vector_f32_ptr;
}