// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#pragma once

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#define MAX_DMA_LOOP 64
#define HALF_TILE_SIZE 24 * 1024
#define TILE_SIZE 48 * 1024
#define MAX_TILE_CNT 100
#define MAX_PIPO_TILE_CNT 50
#define MAX_LOCAL_MEM_SIZE 128 * 1024

// h for horizontal ; v for vertical ; c for channel
typedef struct {
    uint32_t horizontals;
    uint32_t verticals;
    uint32_t tiles;
    // NOTE : different to tensor shape , there's no N(batch dimension) , N is folded to c .
} dma_shape_t;

typedef struct {
    int32_t x1;
    int32_t y1;
    int32_t width;
    int32_t height;
    // NOTE: struct for direct using XICNN ops
} op_tile_coor;

// h for horizontal ; v for vertical ; c for channel
typedef struct {
    uint32_t row_sz;
    uint32_t nrows;
    uint32_t ntiles;

    uint32_t src_row_pitch;
    uint32_t dst_row_pitch;
    uint32_t src_tile_pitch;
    uint32_t dst_tile_pitch;

    uint32_t pixel_bytes;

    char* src_data;
    char* dst_data;

    uint32_t src_offset;
    uint32_t dst_offset;

    uint32_t offset_h;
    uint32_t offset_v;
    uint32_t offset_c;

    uint32_t loop_time_h;
    uint32_t loop_time_v;
    uint32_t loop_time_c;

    uint32_t loop_cnt_h;
    uint32_t loop_cnt_v;
    uint32_t loop_cnt_c;

    dma_shape_t tile_shape;
    dma_shape_t full_shape;

    int32_t tile_on_src_flag;  // flag to describe tile on src or dst .
} dma_param_t;

typedef struct {
    // the offset is in/out data in ddr
    int32_t src_offset;
    int32_t dst_offset;

    int32_t row_buf_sz;
    int32_t nrows;
    int32_t tiles;

    int32_t src_row_buf_pitch;
    int32_t dst_row_buf_pitch;
    int32_t src_tile_buf_pitch;
    int32_t dst_tile_buf_pitch;
} DMA_PARAMS_S;

extern void schedule_dma_nidx(dma_param_t* dma_param, int32_t chn);
extern void schedule_dma(dma_param_t* dma_param, int32_t idx, int32_t chn);
extern uint32_t get_dma_params(char* src_ping, char* src_pong, char* dst_ping, char* dst_pong, dma_shape_t tile_shape,
                               dma_shape_t full_shape, dma_param_t* dma_param, uint32_t src_row_pitch,
                               uint32_t src_tile_pitch, uint32_t dst_row_pitch, uint32_t dst_tile_pitch,
                               int32_t tile_on_src, uint32_t pixel_bytes);
extern char* get_tile_inout_addr(int32_t input_flag, uint32_t (*get_base_fn)());
extern void set_dma_param(dma_param_t* dma_param, char* src, char* dst, uint32_t row_sz, uint32_t nrows,
                          uint32_t ntiles, int32_t tile_on_src, dma_shape_t tile_shape, dma_shape_t full_shape,
                          uint32_t loop_time_c, uint32_t loop_time_h, uint32_t loop_time_v, uint32_t loop_cnt_c,
                          uint32_t loop_cnt_h, uint32_t loop_cnt_v, uint32_t src_row_pitch, uint32_t dst_row_pitch,
                          uint32_t src_tile_pitch, uint32_t dst_tile_pitch, uint32_t pixel_bytes);

void tile_dma_copy(DMA_PARAMS_S cur_param, char* src_data, char* dst_data, int32_t chn);
