// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#pragma once
/**
 * @brief This macro function select four vecN_2xf32 vector
 *        from lut_ptr according the vec_lut_idx input id.
 *
 * @param   vec_lut_idx    The input lut id vector data, data type is vec2Nx8u.
 * @param   lut_ptr  The input lut data address,data type is float32
 *
 * @param   vec_val0  The output lut vector data,data type is vecN_2xf32
 * @param   vec_val1  The output lut vector data,data type is vecN_2xf32
 * @param   vec_val2  The output lut vector data,data type is vecN_2xf32
 * @param   vec_val3  The output lut vector data,data type is vecN_2xf32
 *
 */
#define SEL_DATA_FROM_LUT_S8F32_4OUT(vec_lut_idx, lut_ptr, vec_val0, vec_val1, vec_val2, vec_val3) \
    {                                                                                              \
        xb_vecNx16U vec_idx_h = IVP_UNPKU2NX8_H(vec_lut_idx);                                      \
        xb_vecNx16U vec_idx_l = IVP_UNPKU2NX8_L(vec_lut_idx);                                      \
        vec_idx_h = IVP_SLLINX16U(vec_idx_h, 2);                                                   \
        vec_idx_l = IVP_SLLINX16U(vec_idx_l, 2);                                                   \
                                                                                                   \
        xb_vecN_2x32Uv vec_idx_hh = IVP_UNPKUNX16_H(vec_idx_h);                                    \
        xb_vecN_2x32Uv vec_idx_hl = IVP_UNPKUNX16_L(vec_idx_h);                                    \
                                                                                                   \
        xb_vecN_2x32Uv vec_idx_lh = IVP_UNPKUNX16_H(vec_idx_l);                                    \
        xb_vecN_2x32Uv vec_idx_ll = IVP_UNPKUNX16_L(vec_idx_l);                                    \
                                                                                                   \
        xb_gsr gr0 = IVP_GATHERAN_2XF32(lut_ptr, vec_idx_ll);                                      \
        vec_val0 = IVP_GATHERDN_2XF32(gr0);                                                        \
                                                                                                   \
        xb_gsr gr1 = IVP_GATHERAN_2XF32(lut_ptr, vec_idx_lh);                                      \
        vec_val1 = IVP_GATHERDN_2XF32(gr1);                                                        \
                                                                                                   \
        xb_gsr gr2 = IVP_GATHERAN_2XF32(lut_ptr, vec_idx_hl);                                      \
        vec_val2 = IVP_GATHERDN_2XF32(gr2);                                                        \
                                                                                                   \
        xb_gsr gr3 = IVP_GATHERAN_2XF32(lut_ptr, vec_idx_hh);                                      \
        vec_val3 = IVP_GATHERDN_2XF32(gr3);                                                        \
    }

/**
 * @brief This macro function select four vecN_2xf32 vector with mask
 *        from lut_ptr according the vec_lut_idx input id.
 *
 * @param   vec_lut_idx    The input lut id vector data, data type is vec2Nx8u.
 * @param   lut_ptr        The input lut data address,data type is float32
 * @param   mask           The mask vector indicate the valid data in vec_val3,data type is vboolN_2
 * @param   vec_val0       The output lut vector data,data type is vecN_2xf32
 * @param   vec_val1       The output lut vector data,data type is vecN_2xf32
 * @param   vec_val2       The output lut vector data,data type is vecN_2xf32
 * @param   vec_val3       The output lut vector data,data type is vecN_2xf32
 *
 */
#define SEL_DATA_FROM_LUT_S8F32_4OUT_T(vec_lut_idx, lut_ptr, mask, vec_val0, vec_val1, vec_val2, vec_val3) \
    {                                                                                                      \
        xb_vecNx16 vec_idx_h = IVP_UNPKU2NX8_H(vec_lut_idx);                                               \
        xb_vecNx16 vec_idx_l = IVP_UNPKU2NX8_L(vec_lut_idx);                                               \
                                                                                                           \
        vec_idx_h = IVP_SLLINX16U(vec_idx_h, 2);                                                           \
        vec_idx_l = IVP_SLLINX16U(vec_idx_l, 2);                                                           \
                                                                                                           \
        xb_vecN_2x32Uv vec_idx_hh = IVP_UNPKUNX16_H(vec_idx_h);                                            \
        xb_vecN_2x32Uv vec_idx_hl = IVP_UNPKUNX16_L(vec_idx_h);                                            \
                                                                                                           \
        xb_vecN_2x32Uv vec_idx_lh = IVP_UNPKUNX16_H(vec_idx_l);                                            \
        xb_vecN_2x32Uv vec_idx_ll = IVP_UNPKUNX16_L(vec_idx_l);                                            \
                                                                                                           \
        xb_gsr gr0 = IVP_GATHERAN_2XF32(lut_ptr, vec_idx_ll);                                              \
        vec_val0 = IVP_GATHERDN_2XF32(gr0);                                                                \
                                                                                                           \
        xb_gsr gr1 = IVP_GATHERAN_2XF32(lut_ptr, vec_idx_lh);                                              \
        vec_val1 = IVP_GATHERDN_2XF32(gr1);                                                                \
                                                                                                           \
        xb_gsr gr2 = IVP_GATHERAN_2XF32(lut_ptr, vec_idx_hl);                                              \
        vec_val2 = IVP_GATHERDN_2XF32(gr2);                                                                \
                                                                                                           \
        xb_gsr gr3 = IVP_GATHERAN_2XF32T(lut_ptr, vec_idx_hh, mask);                                       \
        vec_val3 = IVP_GATHERDN_2XF32(gr3);                                                                \
    }

/**
 * @brief This macro function select three vecN_2xf32 vector with mask
 *        from lut_ptr according the vec_lut_idx input id.
 *
 * @param   vec_lut_idx    The input lut id vector data, data type is vec2Nx8u.
 * @param   lut_ptr        The input lut data address,data type is float32
 * @param   mask           The mask vector indicate the valid data in vec_val2,data type is vboolN_2
 * @param   vec_val0       The output lut vector data,data type is vecN_2xf32
 * @param   vec_val1       The output lut vector data,data type is vecN_2xf32
 * @param   vec_val2       The output lut vector data,data type is vecN_2xf32
 *
 */
#define SEL_DATA_FROM_LUT_S8F32_3OUT_T(vec_lut_idx, lut_ptr, mask, vec_val0, vec_val1, vec_val2) \
    {                                                                                            \
        xb_vecNx16 vec_idx_h = IVP_UNPKU2NX8_H(vec_lut_idx);                                     \
        xb_vecNx16 vec_idx_l = IVP_UNPKU2NX8_L(vec_lut_idx);                                     \
                                                                                                 \
        vec_idx_h = IVP_SLLINX16U(vec_idx_h, 2);                                                 \
        vec_idx_l = IVP_SLLINX16U(vec_idx_l, 2);                                                 \
                                                                                                 \
        xb_vecN_2x32Uv vec_idx_hl = IVP_UNPKUNX16_L(vec_idx_h);                                  \
        xb_vecN_2x32Uv vec_idx_lh = IVP_UNPKUNX16_H(vec_idx_l);                                  \
        xb_vecN_2x32Uv vec_idx_ll = IVP_UNPKUNX16_L(vec_idx_l);                                  \
                                                                                                 \
        xb_gsr gr0 = IVP_GATHERAN_2XF32(lut_ptr, vec_idx_ll);                                    \
        vec_val0 = IVP_GATHERDN_2XF32(gr0);                                                      \
                                                                                                 \
        xb_gsr gr1 = IVP_GATHERAN_2XF32(lut_ptr, vec_idx_lh);                                    \
        vec_val1 = IVP_GATHERDN_2XF32(gr1);                                                      \
                                                                                                 \
        xb_gsr gr2 = IVP_GATHERAN_2XF32T(lut_ptr, vec_idx_hl, mask);                             \
        vec_val2 = IVP_GATHERDN_2XF32(gr2);                                                      \
    }
/**
 * @brief This macro function select two vecN_2xf32 vector with mask
 *        from lut_ptr according the vec_lut_idx input id.
 *
 * @param   vec_lut_idx    The input lut id vector data, data type is vec2Nx8u.
 * @param   lut_ptr        The input lut data address,data type is float32
 * @param   mask           The mask vector indicate the valid data in vec_val1,data type is vboolN_2
 * @param   vec_val0       The output lut vector data,data type is vecN_2xf32
 * @param   vec_val1       The output lut vector data,data type is vecN_2xf32
 *
 */
#define SEL_DATA_FROM_LUT_S8F32_2OUT_T(vec_lut_idx, lut_ptr, mask, vec_val0, vec_val1) \
    {                                                                                  \
        xb_vecNx16 vec_idx_l = IVP_UNPKU2NX8_L(vec_lut_idx);                           \
        vec_idx_l = IVP_SLLINX16U(vec_idx_l, 2);                                       \
                                                                                       \
        xb_vecN_2x32Uv vec_idx_lh = IVP_UNPKUNX16_H(vec_idx_l);                        \
        xb_vecN_2x32Uv vec_idx_ll = IVP_UNPKUNX16_L(vec_idx_l);                        \
                                                                                       \
        xb_gsr gr0 = IVP_GATHERAN_2XF32(lut_ptr, vec_idx_ll);                          \
        vec_val0 = IVP_GATHERDN_2XF32(gr0);                                            \
                                                                                       \
        xb_gsr gr1 = IVP_GATHERAN_2XF32T(lut_ptr, vec_idx_lh, mask);                   \
        vec_val1 = IVP_GATHERDN_2XF32(gr1);                                            \
    }
/**
 * @brief This macro function select one vecN_2xf32 vector with mask
 *        from lut_ptr according the vec_lut_idx input id.
 *
 * @param   vec_lut_idx    The input lut id vector data, data type is vec2Nx8u.
 * @param   lut_ptr        The input lut data address,data type is float32
 * @param   mask           The mask vector indicate the valid data in vec_val0,data type is vboolN_2
 * @param   vec_val0       The output lut vector data,data type is vecN_2xf32
 *
 */
#define SEL_DATA_FROM_LUT_S8F32_OUT_T(vec_lut_idx, lut_ptr, mask, vec_val0) \
    {                                                                       \
        xb_vecNx16 vec_idx_l = IVP_UNPKU2NX8_L(vec_lut_idx);                \
        vec_idx_l = IVP_SLLINX16U(vec_idx_l, 2);                            \
        xb_vecN_2x32Uv vec_idx_ll = IVP_UNPKUNX16_L(vec_idx_l);             \
        xb_gsr gr0 = IVP_GATHERAN_2XF32T(lut_ptr, vec_idx_ll, mask);        \
        vec_val0 = IVP_GATHERDN_2XF32(gr0);                                 \
    }

/**
 * @brief This macro function read data from input buffer,and convert the data into lut index.
 *
 * @param   src_vptr          The input vector data ptr, data type is xb_vec2Nx8S*
 * @param   vec_input         The input vector data,data type is xb_vec2Nx8S
 * @param   va_src            The input align vector
 * @param   vec_lut_idx       The output lut index vector,data type is xb_vec2Nx8U
 *
 */
#define GET_LUT_ID_FROM_INPUT(src_vptr, vec_input, va_src, vec_lut_idx) \
    {                                                                   \
        va_src = IVP_LA2NX8_PP(src_vptr);                               \
        IVP_LA2NX8_IP(vec_input, va_src, src_vptr);                     \
        vec_input = IVP_SUB2NX8(vec_input, max_val);                    \
        vec_lut_idx = IVP_ADD2NX8(vec_input, 255);                      \
    }

/**
 * @brief This macro function select two vecNxf16 vector
 *        from lut_ptr according the vec_lut_idx input id.
 *
 * @param   vec_lut_idx    The input lut id vector data, data type is vec2Nx8u.
 * @param   lut_ptr        The input lut data address,data type is float16
 * @param   vec_val0       The output lut vector data,data type is vecNxf16
 * @param   vec_val1       The output lut vector data,data type is vecNxf16
 *
 */
#define SEL_DATA_FROM_LUT_S8F16_2OUT(vec_lut_idx, lut_ptr, vec_val0, vec_val1) \
    {                                                                          \
        xb_vecNx16U vec_idx_h = IVP_UNPKU2NX8_H(vec_lut_idx);                  \
        xb_vecNx16U vec_idx_l = IVP_UNPKU2NX8_L(vec_lut_idx);                  \
        vec_idx_h = IVP_SLLINX16U(vec_idx_h, 1);                               \
        vec_idx_l = IVP_SLLINX16U(vec_idx_l, 1);                               \
                                                                               \
        xb_gsr gr0 = IVP_GATHERANXF16(lut_ptr, vec_idx_l);                     \
        vec_val0 = IVP_GATHERDNXF16(gr0);                                      \
                                                                               \
        xb_gsr gr1 = IVP_GATHERANXF16(lut_ptr, vec_idx_h);                     \
        vec_val1 = IVP_GATHERDNXF16(gr1);                                      \
    }

/**
 * @brief This macro function select two vecNxf16 vector with mask
 *        from lut_ptr according the vec_lut_idx input id.
 *
 * @param   vec_lut_idx    The input lut id vector data, data type is vec2Nx8u.
 * @param   lut_ptr        The input lut data address,data type is float16
 * @param   mask           The mask vector indicate the valid data in vec_val1,data type is vboolN
 * @param   vec_val0       The output lut vector data,data type is vecNxf16
 * @param   vec_val1       The output lut vector data,data type is vecNxf16
 *
 */
#define SEL_DATA_FROM_LUT_S8F16_2OUT_T(vec_lut_idx, lut_ptr, mask, vec_val0, vec_val1) \
    {                                                                                  \
        xb_vecNx16 vec_idx_h = IVP_UNPKU2NX8_H(vec_lut_idx);                           \
        xb_vecNx16 vec_idx_l = IVP_UNPKU2NX8_L(vec_lut_idx);                           \
                                                                                       \
        vec_idx_h = IVP_SLLINX16U(vec_idx_h, 1);                                       \
        vec_idx_l = IVP_SLLINX16U(vec_idx_l, 1);                                       \
                                                                                       \
        xb_gsr gr0 = IVP_GATHERANXF16(lut_ptr, vec_idx_l);                             \
        vec_val0 = IVP_GATHERDNXF16(gr0);                                              \
                                                                                       \
        xb_gsr gr1 = IVP_GATHERANXF16T(lut_ptr, vec_idx_h, mask);                      \
        vec_val1 = IVP_GATHERDNXF16(gr1);                                              \
    }

/**
 * @brief This macro function select one vecNxf16 vector with mask
 *        from lut_ptr according the vec_lut_idx input id.
 *
 * @param   vec_lut_idx    The input lut id vector data, data type is vec2Nx8u.
 * @param   lut_ptr        The input lut data address,data type is float16
 * @param   mask           The mask vector indicate the valid data in vec_val1,data type is vboolN
 * @param   vec_val0       The output lut vector data,data type is vecNxf16
 *
 */
#define SEL_DATA_FROM_LUT_S8F16_OUT_T(vec_lut_idx, lut_ptr, mask, vec_val0) \
    {                                                                       \
        xb_vecNx16 vec_idx_l = IVP_UNPKU2NX8_L(vec_lut_idx);                \
                                                                            \
        vec_idx_l = IVP_SLLINX16U(vec_idx_l, 1);                            \
                                                                            \
        xb_gsr gr0 = IVP_GATHERANXF16T(lut_ptr, vec_idx_l, mask);           \
        vec_val0 = IVP_GATHERDNXF16(gr0);                                   \
    }

/**
 * @brief This macro function select two vecNxs16 vector
 *        from lut_ptr according the vec_lut_idx input id.
 *
 * @param   vec_lut_idx    The input lut id vector data, data type is vec2Nx8u.
 * @param   lut_ptr        The input lut data address,data type is int16
 * @param   vec_val0       The output lut vector data,data type is vecNxs16
 * @param   vec_val1       The output lut vector data,data type is vecNxs16
 *
 */
#define SEL_DATA_FROM_LUT_S8S16_2OUT(vec_lut_idx, lut_ptr, vec_val0, vec_val1) \
    {                                                                          \
        xb_vecNx16U vec_idx_h = IVP_UNPKU2NX8_H(vec_lut_idx);                  \
        xb_vecNx16U vec_idx_l = IVP_UNPKU2NX8_L(vec_lut_idx);                  \
        vec_idx_h = IVP_SLLINX16U(vec_idx_h, 1);                               \
        vec_idx_l = IVP_SLLINX16U(vec_idx_l, 1);                               \
                                                                               \
        xb_gsr gr0 = IVP_GATHERANX16(lut_ptr, vec_idx_l);                      \
        vec_val0 = IVP_GATHERDNX16(gr0);                                       \
                                                                               \
        xb_gsr gr1 = IVP_GATHERANX16(lut_ptr, vec_idx_h);                      \
        vec_val1 = IVP_GATHERDNX16(gr1);                                       \
    }

/**
 * @brief This macro function select two vecNxs16 vector with mask
 *        from lut_ptr according the vec_lut_idx input id.
 *
 * @param   vec_lut_idx    The input lut id vector data, data type is vec2Nx8u.
 * @param   lut_ptr        The input lut data address,data type is int16
 * @param   mask0          The mask vector indicate the valid data in vec_val0,data type is vboolN
 * @param   mask0          The mask vector indicate the valid data in vec_val1,data type is vboolN
 * @param   vec_val0       The output lut vector data,data type is vecNxs16
 * @param   vec_val1       The output lut vector data,data type is vecNxs16
 *
 */
#define SEL_DATA_FROM_LUT_S8S16_2OUT_T(vec_lut_idx, lut_ptr, mask0, mask1, vec_val0, vec_val1) \
    {                                                                                          \
        xb_vecNx16U vec_idx_h = IVP_UNPKU2NX8_H(vec_lut_idx);                                  \
        xb_vecNx16U vec_idx_l = IVP_UNPKU2NX8_L(vec_lut_idx);                                  \
        vec_idx_h = IVP_SLLINX16U(vec_idx_h, 1);                                               \
        vec_idx_l = IVP_SLLINX16U(vec_idx_l, 1);                                               \
                                                                                               \
        xb_gsr gr0 = IVP_GATHERANX16T(lut_ptr, vec_idx_l, mask0);                              \
        vec_val0 = IVP_GATHERDNX16(gr0);                                                       \
                                                                                               \
        xb_gsr gr1 = IVP_GATHERANX16T(lut_ptr, vec_idx_h, mask1);                              \
        vec_val1 = IVP_GATHERDNX16(gr1);                                                       \
    }
