// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef TRANS_NHW3_TO_N3HW_H
#define TRANS_NHW3_TO_N3HW_H

ES_S32 trans_nhw3_to_n3hw(int8_t *dst_ptr, int8_t *src_ptr, int32_t h, int32_t w)
{
    // step 1: calc tile h
    const int32_t c = 3;
    const int32_t dram_buf_size = 64 * 1024;
    int32_t max_tile_h = dram_buf_size / c / w;
    int32_t tile_h = max_tile_h;
    int32_t tile_num = 0;
    while (tile_h > 0) {
        if (h % tile_h == 0) {
            tile_num = h / tile_h;
            break;
        } else {
            tile_h--;
        }
    }

    int8_t *dram0_st_ptr = (int8_t *)get_available_dram0_base();
    int8_t *dram1_st_ptr = (int8_t *)get_available_dram1_base();
    for (int tile_cnt_i = 0; tile_cnt_i < tile_num; ++tile_cnt_i) {
        // step 2: cp src_ptr from ddr to dram
        int8_t *cur_src_ddr_ptr = src_ptr + tile_cnt_i * tile_h * w * c;
        int8_t *cur_dst_ddr_ptr = dst_ptr + tile_cnt_i * tile_h * w;
        idma_copy_3d_desc64(0, &dram0_st_ptr, &cur_src_ddr_ptr, IDMA_DISABLE_INT, tile_h * w * c, 1, 1, 1, 1, 1, 1);
        WAIT_DMA(0)

        // step 3: trans tile_h w 3 --> 3 tile_h w
        int8_t *ifmap_ptr = dram0_st_ptr;
        int8_t *ofmap_ptr = dram1_st_ptr;

        xb_vecNx16U vec_offset = IVP_MULNX16UPACKL(IVP_SEQNX16U(), c);
        int8_t *__restrict p_src;
        int8_t *__restrict p_c_src;
        int8_t *__restrict p_ch_src;
        xb_vecNx8 *__restrict p_dst = (xb_vecNx8 *)(ofmap_ptr);
        xb_vecNx16 vec_out;
        valign va_dst = IVP_ZALIGN();
        xb_gsr gr0;
        int32_t c_i, w_i;
        int32_t rem_x;
        for (c_i = 0; c_i < 3; c_i++) {
            p_c_src = ifmap_ptr + c_i;
            for (int h_i = 0; h_i < tile_h; ++h_i) {
                p_ch_src = p_c_src + h_i * w * c;
                for (w_i = 0; w_i < w; w_i += XCHAL_IVPN_SIMD_WIDTH) {
                    rem_x = XT_MIN((w - w_i), XCHAL_IVPN_SIMD_WIDTH);
                    p_src = p_ch_src + w_i * c;

                    gr0 = IVP_GATHERANX8S(p_src, vec_offset);
                    vec_out = IVP_GATHERDNX8S(gr0);
                    IVP_SAVNX8S_XP(vec_out, va_dst, p_dst, rem_x);
                }
                IVP_SAPOSNX8S_FP(va_dst, p_dst);
            }
        }

        // step 4: cp dst_ptr from dram to ddr
        idma_copy_3d_desc64(0, &cur_dst_ddr_ptr, &dram1_st_ptr, IDMA_DISABLE_INT, tile_h * w, c, 1, tile_h * w, h * w,
                            1, 1);
        WAIT_DMA(0)
    }

    return 0;
}

#endif  // TRANS_NHW3_TO_N3HW_H
