// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#pragma once

//#define VEC_PROBE

#ifdef VEC_PROBE
#include "stdint.h"
#include "stdio.h"
#include "stdlib.h"
#include <xtensa/tie/xt_ivpn.h>
#define _LOCAL_DRAM0_ __attribute__((section(".dram0.data")))
#define _LOCAL_DRAM1_ __attribute__((section(".dram1.data")))

char _LOCAL_DRAM0_ probeMemS8[64];
int16_t _LOCAL_DRAM0_ probeMemS16[32];
int32_t _LOCAL_DRAM0_ probeMemS32[16];
xb_f16 _LOCAL_DRAM0_ probeMemF16[32];
float probeMemF16F32[32];

static void setF16F32()
{
    for (int i = 0; i < 32; ++i) {
        probeMemF16F32[i] = (float)probeMemF16[i];
        printf("%f ", probeMemF16F32[i]);
    }
    printf("\n");
}
#define PROBENX8(x)                                     \
    {                                                   \
        xb_vecNx16U scaterOffset = IVP_SEQNX16U() << 0; \
        IVP_SCATTERNX8U(x, probeMemS8, scaterOffset);   \
    }

#define PROBE2NX8(x)                                         \
    {                                                        \
        xb_vecNx16U scaterOffset = IVP_SEQNX16U() << 0;      \
        IVP_SCATTER2NX8_L(x, probeMemS8, scaterOffset);      \
        IVP_SCATTER2NX8_H(x, probeMemS8 + 32, scaterOffset); \
    }

#define PROBENX16(x)                                    \
    {                                                   \
        xb_vecNx16U scaterOffset = IVP_SEQNX16U() << 1; \
        IVP_SCATTERNX16(x, probeMemS16, scaterOffset);  \
    }

#define PROBEN_2X32(x)                                       \
    {                                                        \
        xb_vecN_2x32Uv scaterOffset = IVP_SEQN_2X32U() << 2; \
        IVP_SCATTERN_2X32(x, probeMemS32, scaterOffset);     \
    }

#define PROBENXF16(x)                                   \
    {                                                   \
        xb_vecNx16U scaterOffset = IVP_SEQNX16U() << 1; \
        IVP_SCATTERNXF16(x, probeMemF16, scaterOffset); \
        setF16F32();                                    \
    }

int32_t get_n_2x_widx(xb_vecN_2x32v x, int32_t idx)
{
    PROBEN_2X32(x);
    for (int i = 0; i < 16; ++i) {
        if (idx == i) return probeMemS32[i];
    }
}

int8_t get_nx8_widx(xb_vecNx8U x, int32_t idx)
{
    PROBENX8(x);
    for (int i = 0; i < 32; ++i) {
        if (idx == i) return probeMemS8[i];
    }
}

void print_nx16(xb_vecNx16U x, char TAG[])
{
    PROBENX16(x);
    printf(" %s ", TAG);
    for (int i = 0; i < 32; ++i) {
        printf("%d,", probeMemS16[i]);
    }
    printf("\n");
}
void print_n_x8(xb_vecNx8U x, char TAG[])
{
    PROBENX8(x);
    printf(" %s ", TAG);
    for (int i = 0; i < 32; ++i) {
        printf("%d,", probeMemS8[i]);
    }
    printf("\n");
}
void print_n_2x32(xb_vecN_2x32v x, char TAG[])
{
    PROBEN_2X32(x);
    printf(" %s ", TAG);
    for (int i = 0; i < 16; ++i) {
        printf("%d,", probeMemS32[i]);
    }
    printf("\n");
}

#else
#define PROBE2NX8(x) \
    {                \
    }
#define PROBENX16(x) \
    {                \
    }
#define PROBENXF16(x) \
    {                 \
    }
#endif
