/*
 * Copyright (c) 2019-2022 by Cadence Design Systems. ALL RIGHTS RESERVED.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <stdlib.h>
#include <xt_mld_loader.h>
#include "interface.h"

extern "C"
{
void * _start(struct app_symbol_tray * ex);
}

XTMLD_LIBRARY_NAME (mult_section_cpp_lib);

/*
 * Here we define the preference (requirements) for the loading the code
 * sections : .text. .1.text, .2.text, and .3.text
 *
 * Code with minimal performance impact ==> xtmld_load_normal
 * Performance sensitive code           ==> xtmld_load_prefer_localmem
 * Requirement for local mem            ==> xtmld_load_require_localmem
 */
XTMLD_CODE_MEM_REQ(xtmld_load_prefer_localmem, xtmld_load_prefer_localmem,
        xtmld_load_normal, xtmld_load_normal);

/*
 * Here we define the preference (requirements) for the loading the data
 * sections : .data, .1.data, .2.data, and .3.data
 *
 * Data with minimal performance impact ==> xtmld_load_normal
 * Performance sensitive data           ==> xtmld_load_prefer_localmem
 * Requirement for local mem            ==> xtmld_load_require_localmem
 */
XTMLD_DATA_MEM_REQ(xtmld_load_normal, xtmld_load_prefer_localmem,
        xtmld_load_prefer_localmem, xtmld_load_normal);


static struct app_symbol_tray* app_tray;

static struct cpplib_symbol_tray cpplib_tray;


__attribute__ ((section (".data"))) static int b[] =
{ 1, 2, 3, 4, 5, 6, 7, 8, 9 };

__attribute__ ((section (".2.data"))) static int c[] =
{ 2, 4, 5, 6 };

class b
{
public:
    const char* zz;
    b(const char* z) :
            zz(z)
    {
    }
    ~b()
    {
        app_tray->printf("static destructor called %s\n", zz);
    }
};

__attribute((section(".3.data")))   static class b z3("in .3.data");
__attribute((section(".2.data")))   static class b z2("in .2.data");
__attribute((section(".1.data")))   static class b z1("in .1.data");
__attribute((section(".data")))   static class b z0("in .data");

__attribute__((section (".text"))) void my_other_exit()
{
    app_tray->printf("other atexit() function successfully called\n");
}

__attribute__((section (".3.text"))) void my_exit()
{
    app_tray->printf("atexit() function successfully called\n");
}

/* This function is called from the main application. */
__attribute__((section (".1.text"))) const char * foo(void)
{
    /* register atexit() functions */
    atexit(my_exit);
    atexit(my_other_exit);
    app_tray->printf("executing function foo\n");
    app_tray->printf("Data in .data, should be [1, 5, 9]] = [%d %d %d]\n", b[0], b[4],
            b[8]);
    app_tray->printf("Data in .2.data, should be [2, 6] = [%d %d]\n", c[0], c[3]);
    app_tray->printf("class b z0 %s\n", z0.zz);
    app_tray->printf("class b z1 %s\n", z1.zz);
    app_tray->printf("class b z2 %s\n", z2.zz);
    app_tray->printf("class b z3 %s\n", z3.zz);

    return "this string returned from foo";
}

/* This function is called from the main application. */
__attribute__((section (".1.text"))) double otherfoo(const char* a, int b)
{
    app_tray->printf("a = %s\n", a);
    return b * b;
}


/* This function is called from the main application. */
__attribute__((section(".2.text"))) void * _start(struct app_symbol_tray * ex)
{
    app_tray = ex;
    cpplib_tray.foo = foo;
    cpplib_tray.otherfoo = otherfoo;
    /* return a pointer to function foo() */
    return &cpplib_tray;
}
