/*
 * Copyright (c) 2019-2022 by Cadence Design Systems. ALL RIGHTS RESERVED.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef _interface_h_
#define _interface_h_

#include <stdint.h>

/*
 * A tray to define all symbols in the application that should be
 * visible in the library.  A pointer to this tray is passed as an
 * argument to the start function.
 *
 * This is useful to pass pointers to certain standard library functions
 * that otherwise would be unusable from within the library.
 */
struct app_symbol_tray
{
  int   (*printf)(const char*, ...);
  void  (*abort)(void);
  void* (*malloc)(size_t);
  void  (*free)(void*);
};

/* A tray to define all external interfaces of the library.
 * Any symbol that should be externally accessible is defined in this
 * structure.  A pointer to this structure is returned from that _start function
 * giving the application program a handle to call function or access variables
 * in the library.
 */
struct lib_symbol_tray
{
    const char* (*foo)(void);
    double (*foo2)(int);
};

struct cpplib_symbol_tray
{
    const char* (*foo)(void);
    double (*otherfoo)(const char*, int);
};

#endif
